/*!
 * vue-form-container v0.0.0
 * (c) 2018-present Vitor Luiz Cavalcanti <vitorluizc@outlook.com> (https://vitorluizc.github.io)
 * Released under the MIT License.
 */
import { isValid, validate, validateSchema } from 'valite';

/**
 * A function to check if value attends to predicate.
 * @callback Predicate
 * @param {*} value Any value could be checked.
 * @returns {boolean} `true` if value attends to predicate.
 */

/**
 * Value returned by `typeof` to determine what is value's type.
 * @typedef {('number' | 'object' | 'string' | 'symbol' | 'boolean' | 'function' | 'undefined')} Type
 */

/**
 * Check if value is of type.
 * @param {Type} type
 * @returns {Predicate}
 */
var isType = function (type) { return function (value) { return typeof value === type; }; };
/**
 * Check if value is an object.
 * @param {*} value
 * @returns {boolean}
 */

var isObject = function (value) { return !!value && isType('object')(value); };
/**
 * Check if value is not an empty string.
 * @param {*} value
 * @returns {boolean}
 */

var isNotEmptyString = function (value) { return isType('string')(value) && !!value.trim(); };
/**
 * Check if value is an Array and every element attends the predicate.
 * @param {Predicate} predicate
 * @returns {Predicate}
 */

var isEvery = function (predicate) { return function (value) { return Array.isArray(value) && value.every(predicate); }; };
/**
 * Check if every object properties attends the predicate.
 * @param {Predicate} predicate
 * @returns {Predicate}
 */

var isEveryProperty = function (predicate) { return function (value) {
  if (!isObject(value)) { return false; }
  var properties = Object.keys(value).map(function (key) { return value[key]; });
  return isEvery(predicate)(properties);
}; };

//
var script = {
  props: {
    name: {
      type: String,
      required: true,
      validator: isNotEmptyString
    },
    model: {
      type: Object,
      required: true,
      validator: isEveryProperty(isType('function'))
    }
  },

  data: function data() {
    return {
      ticks: 0,
      errors: Object.create(null),
      values: Object.create(null)
    };
  },

  watch: {
    name: {
      handler: 'rename',
      immediate: true
    }
  },
  computed: {
    fields: function fields() {
      var this$1 = this;

      var descriptors = {};
      Object.keys(this.model).forEach(function (name) {
        descriptors[name] = {
          get: function () { return this$1.values[name]; },
          set: function (value) { return this$1.update(name, value); }
        };
      });
      return Object.create(null, descriptors);
    },

    isLoading: function isLoading() {
      this.ticks > 0;
    },

    isValid: function isValid$1() {
      return isValid(this.errors) && this.isLoading;
    }

  },
  methods: {
    /**
     * When name is change it swiches registered form name.
     * @param {string} to
     * @param {string} [from]
     */
    rename: function rename(to, from) {
      if (isNotEmptyString(from)) { this.$form.unregister(from); }
      this.$form.register(to, this);
    },

    /**
     * Updates a field and trigger its validation.
     * @param {string} field Field's name.
     * @param {*} value Field's value.
     */
    update: function update(field, value) {
      this.$set(this.values, field, value);
      this.validateField(field);
    },

    /**
     * Validates a field and updates Form properties.
     * @param {string} field Field's name.
     * @returns {Promise.<void>}
     */
    validateField: function validateField(field) {
      return new Promise(function ($return, $error) {
        var value, validators;
        value = this.values[field];
        validators = this.model[field];

        var $Try_1_Post = function () {
          try {
            return $return();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        var $Try_1_Catch = function (error) {
          try {
            this.ticks -= 1;
            console.error(("Can't validate field \"" + field + "\"."));
            return $Try_1_Post();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        }.bind(this);

        try {
          var error$1;
          this.ticks += 1;
          return Promise.resolve(validate(value, validators)).then(function ($await_3) {
            try {
              error$1 = $await_3;
              this.$set(this.errors, field, error$1);
              this.ticks -= 1;
              return $Try_1_Post();
            } catch ($boundEx) {
              return $Try_1_Catch($boundEx);
            }
          }.bind(this), $Try_1_Catch);
        } catch (error) {
          $Try_1_Catch(error);
        }
      }.bind(this));
    },

    /**
     * Validates all fields and updates Form properties.
     * @returns {Promise.<void>}
     */
    validateForm: function validateForm() {
      return new Promise(function ($return, $error) {
        var $Try_2_Post = function () {
          try {
            return $return();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        };

        var $Try_2_Catch = function (error) {
          try {
            this.ticks -= 1;
            console.error('Can\'t validate form.');
            return $Try_2_Post();
          } catch ($boundEx) {
            return $error($boundEx);
          }
        }.bind(this);

        try {
          var errors;
          this.ticks += 1;
          return Promise.resolve(validateSchema(this.values, this.model)).then(function ($await_4) {
            try {
              errors = $await_4;
              this.errors = errors;
              this.ticks -= 1;
              return $Try_2_Post();
            } catch ($boundEx) {
              return $Try_2_Catch($boundEx);
            }
          }.bind(this), $Try_2_Catch);
        } catch (error) {
          $Try_2_Catch(error);
        }
      }.bind(this));
    }

  },

  beforeDestroy: function beforeDestroy() {
    this.$form.unregister(this.name);
  }

};

/* script */
            var __vue_script__ = script;
            
/* template */
var __vue_render__ = function() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c(
    "form",
    [
      _vm._t("default", null, {
        values: _vm.values,
        errors: _vm.errors,
        fields: _vm.fields,
        isValid: _vm.isValid,
        isLoading: _vm.isLoading,
        validateForm: _vm.validateForm,
        validateField: _vm.validateField
      })
    ],
    2
  )
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

  /* style */
  var __vue_inject_styles__ = undefined;
  /* scoped */
  var __vue_scope_id__ = undefined;
  /* module identifier */
  var __vue_module_identifier__ = undefined;
  /* functional template */
  var __vue_is_functional_template__ = false;
  /* component normalizer */
  function __vue_normalize__(
    template, style, script$$1,
    scope, functional, moduleIdentifier,
    createInjector, createInjectorSSR
  ) {
    var component = (typeof script$$1 === 'function' ? script$$1.options : script$$1) || {};

    // For security concerns, we use only base name in production mode.
    component.__file = "/Users/vitorcavalcanti/Projects/vue-form-container/src/FormContainer.vue";

    if (!component.render) {
      component.render = template.render;
      component.staticRenderFns = template.staticRenderFns;
      component._compiled = true;

      if (functional) { component.functional = true; }
    }

    component._scopeId = scope;

    

    return component
  }
  /* style inject */
  
  /* style inject SSR */
  

  
  var FormContainer = __vue_normalize__(
    { render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ },
    __vue_inject_styles__,
    __vue_script__,
    __vue_scope_id__,
    __vue_is_functional_template__,
    __vue_module_identifier__,
    undefined,
    undefined
  )

/**
 * Globally register FormContainer and it's helpers on Vue application.
 * @param {Vue} Vue
 */

var install = function (Vue) {
  var forms = Object.create(null);
  /**
   * Reaches a registered FormContainer.
   * @param {string} name
   */

  Vue.prototype.$form = function (name) {
    if (!name in forms) { throw new Error(("Can't find form \"" + name + "\".")); }
    return forms[name];
  };
  /**
   * Register a form to use it's helpers on every component.
   * @param {string} name Name used to set form and it's functions.
   * @param {FormContainer} form
   */


  Vue.prototype.$form.register = function (name, form) {
    forms[name] = form;
  };
  /**
   * Unregister form.
   * @param {string} name
   */


  Vue.prototype.$form.unregister = function (name) {
    delete forms[name];
  };

  Vue.component('FormContainer', FormContainer);
};

export default install;
